/* Train Manager */

#include<iostream.h>
#include<fstream.h>
#include<windows.h>
#include<stdio.h>
#include<time.h>

#include"..\TOS CLIENT API\track.h"
#include"..\TOS CLIENT API\kmp_client_api.h"
#include"..\TOS CLIENT API\tos_api.h"


char MyClientName[] = "ResourceMan";
char MyPipeName[]   = "\\\\.\\pipe\\ResourceMan";
char LOGFILE[] = "ResourceMan.log";


const MAXUSERS = 20;
const CYCLE_SLEEP = 100;



void WriteLog(char * message);
bool RequestResource(Track * track, char * data, unsigned long length, char * from, HANDLE pipe);
bool ReleaseResource(Track * myTrack, char * data, unsigned long length, HANDLE pipe);
bool SendResourceData(Track * track, char users[][CLIENT_SIZE_CLIENTNAME], unsigned long usercnt, HANDLE pipe);


void main()
{

	/* Train Manager Startup

	/* Connect to the server */

	HANDLE pipe = ConnectToServer(MyPipeName);
	
	if(pipe == INVALID_HANDLE_VALUE)
	{
		WriteLog("Could Not Connect, exiting.\n");
		return;
	}



	/* Track Data */
	Track myTrack;



	/* Array for users registered for updates */
	char users[MAXUSERS][CLIENT_SIZE_CLIENTNAME];
	unsigned long usercnt = 0;



	/* Post Request For Track Data First */
	TOS_RequestTrackData(pipe);
	TOS_RequestTrackDataUpdates(pipe);

	/* some common variables */
	char From[CLIENT_SIZE_CLIENTNAME];
	unsigned long ServiceCode;
	char MessageData[CLIENT_SIZE_MESSAGE];
	unsigned long MessageLength;


	char logmessage[255];

	/* StartMainLoop */
	while(true)
	{

		Sleep(CYCLE_SLEEP);
		/* Get Next Message From Queue */
		int ret = GetMessageFromPeer(From, ServiceCode, MessageData, MessageLength, pipe);

		if(ret == -2)
		{
			/* Error reading / writing to pipe, pipe broken or server closed */
			return;
		}

		if(ret >= 0)
		{

			/*DETERMINE MESSAGE TYPE*/
			switch(ServiceCode)
			{
			case SC_TRACK_DATA:
				TOS_ReadTrackData(&myTrack, MessageData, MessageLength);
				SendResourceData(&myTrack, users, usercnt, pipe);
				break;
			case SC_REQUEST_OWNERSHIP:
				RequestResource(&myTrack, MessageData, MessageLength, From, pipe);
				SendResourceData(&myTrack, users, usercnt, pipe);
				break;
			case SC_RELEASE_OWNERSHIP:
				ReleaseResource(&myTrack, MessageData, MessageLength, pipe);
				SendResourceData(&myTrack, users, usercnt, pipe);
				break;
			case SC_REQUEST_RESOURCE_UPDATES:
				if(usercnt < MAXUSERS)
				{
					strncpy(users[usercnt], From, CLIENT_SIZE_CLIENTNAME);
					usercnt++;
				}
				break;


			default:
				break;
			}
		}
	}

}

bool SendResourceData(Track * track, char users[][CLIENT_SIZE_CLIENTNAME], unsigned long usercnt, HANDLE pipe)
{

	unsigned long sections;
	unsigned long nodes;

	sections = track->GetSectionCount();
	nodes = track->GetNodeCount();

	Resource * res = new Resource[sections+nodes];

	unsigned long i;
	for(i=0;i<sections; i++)
	{
		Section sec = track->GetSectionByIndex(i);
		res[i].active = sec.Active;
		res[i].id = sec.ID;
		res[i].owner = sec.OwnerID;
		res[i].type = 1;
	}

	for(i=0;i<nodes; i++)
	{
		Node node = track->GetNodeByIndex(i);
		res[i+sections].active = node.Active;
		res[i+sections].id = node.ID;
		res[i+sections].owner = node.OwnerID;
		res[i+sections].type = 2;
	}

	return TOS_SendResourceData(sections + nodes, res, users, usercnt, pipe);

}


bool ReleaseResource(Track * myTrack, char * data, unsigned long length, HANDLE pipe)
{
		/* message format*/
	/*[type(x1)][id(x2)]*/
	unsigned long id;
	unsigned char type;

	if(length != 3)
	{
		WriteLog("Release Resource Message is wrong length.");
		return false;
	}

	type = data[0];

	id = data[1] * 256;
	id += data[2];

	if(type == 1)
	{
		/* section */
		if(myTrack->ClearSectionOwner(id))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	else if(type == 2)
	{
		/* node */
		if(myTrack->ClearNodeOwner(id))
		{
			return true;
		}
		else
		{
			return false;
		}
	}
	else
	{
		/* invalid type */
		return false;
	}
	
}

bool RequestResource(Track * track, char * data, unsigned long length, char * from, HANDLE pipe)
{
		/* Message Format */
	/* [TrainID(x2)][Type(x1)][ResourceID(x2)] */

	/* 1 - is section */
	/* 2 - is node    */

	unsigned long train;
	unsigned char type;
	unsigned long resource;

	if(length != 5)
	{
		WriteLog("Request Resource Message is wrong length.");
	}

	train = data[0] * 256;
	train += data[1];

	type = data[2];

	resource = data[3] * 256;
	resource += data[4];

	if(type == 1)
	{
		/* SECTION */
		Section sec = track->GetSectionById(resource);
		if(sec.ID == 0)
		{
			/* section not found */
			return TOS_DenyResource(train, type, resource, from, pipe);
		}

		if(sec.OwnerID == 0xffffffff) /* no owner */
		{
			track->SetSectionOwner(resource, train);
			return TOS_GrantResource(train, type, resource, from, pipe);
		
		}
		else 
		{
			/* already owned */
			return TOS_DenyResource(train, type, resource, from, pipe);
		}
	}
	else if(type == 2)
	{
		/* NODE */
		Node n = track->GetActiveNode(resource);
		if(n.ID == 0)
		{
			/* node not found */
			return TOS_DenyResource(train, type, resource, from, pipe);
		}
		if(n.OwnerID == 0xffffffff) /* no owner */
		{
			track->SetNodeOwner(resource, train);
			return TOS_GrantResource(train, type, resource, from, pipe);
		
		}
		else 
		{
			/* already owned */
			return TOS_DenyResource(train, type, resource, from, pipe);
		}
	}
	else
	{
		/* INVALID TYPE */
		char message[50];
		sprintf(message, "Id %i cannot get resource %i of type %i because the type is invalid.", &train, &resource, &type);
		WriteLog(message);
		return TOS_DenyResource(train, type, resource, from, pipe);
	}

}



void WriteLog(char * message)
{

	char LogFile[255];
	strcpy(LogFile, LOGFILE);

	ofstream log;
	log.open(LogFile,ios::app);

	cerr << message << "\n";

	if(!log)
	{
		cerr << "Failed to write to log.\n";
		return;
	}
    char dbuffer [9];
    char tbuffer [9];
    _strdate( dbuffer );
    _strtime( tbuffer );
    

	log << dbuffer << " " << tbuffer << " - " << message << "\n";

	return;

}