/***************************************************/
/* API CALLS SPECIFIC TO THE TOS SYSTEM            */
/*                                                 */
/*                                                 */
/*                                                 */
/*                                                 */
/*                                                 */
/*                                                 */
/*                                                 */
/*                                                 */
/*  CHRISTOPHER KRATZ SNC 4/4/2007                 */
/***************************************************/
/* TOS_CLIENT.CPP */

#include<windows.h>
#include"track.h"
#include"kmp_client_api.h"
#include"tos_api.h"

/* TRACK MANAGER RELATED FUNCTIONS */

bool TOS_RequestTrackData(HANDLE pipe)
{
	return SendMessageToPeer(NULL, 0, SC_REQUEST_TRACK_DATA, TRACK_MAN, pipe);
}

bool TOS_RequestTrackDataUpdates(HANDLE pipe)
{
	return SendMessageToPeer(NULL, 0, SC_REQUEST_TRACK_DATA_UPDATES, TRACK_MAN, pipe);
}
	
bool TOS_RequestLoadTrackFromFile(char * filename, HANDLE pipe)
{
	/* Message Format */
	/* [Filename]0x00 */
	if(filename == NULL)
		return false;
		
	int len = strlen(filename);
	
	return SendMessageToPeer(filename, len, SC_LOAD_TRACK_FROM_FILE, TRACK_MAN, pipe);
}
	
bool TOS_SendTrackData(Track * track, char * destination, HANDLE pipe)
{
	/* Message Format */
	/* [# of sections (x4)][#of nodes (x4)][#of sensors(x4)][Sections].... [Nodes]....[sensors]...  */
	if(track==NULL)
		return false;
		
	if(destination == NULL)
		return false;
		
	char * message = new char[track->GetSectionCount() * sizeof(Section) + track->GetNodeCount() * sizeof(Node) + track->GetSensorCount() * sizeof(Sensor) + 16];
	
	if(message == NULL)
		return false;
		
	unsigned long offset = 0;
	
	message[0] = (char)(track->GetSectionCount() / 0x01000000);
	message[1] = (char)(track->GetSectionCount() / 0x00010000);
	message[2] = (char)(track->GetSectionCount() / 0x00000100);
	message[3] = (char)(track->GetSectionCount() % 0x00000100);
	
	message[4] = (char)(track->GetNodeCount() / 0x01000000);
	message[5] = (char)(track->GetNodeCount() / 0x00010000);
	message[6] = (char)(track->GetNodeCount() / 0x00000100);
	message[7] = (char)(track->GetNodeCount() % 0x00000100);

	message[8] = track->GetSensorCount() / 0x01000000;
	message[9] = track->GetSensorCount() / 0x00010000;
	message[10] = track->GetSensorCount() / 0x00010000;
	message[11] = track->GetSensorCount() % 0x00000100;
	
	offset=12;
	
	unsigned long i;
	for(i=0;i<track->GetSectionCount();i++)
	{
		unsigned long j;
		for(j=0;j<sizeof(Section);j++)
		{
			Section sec = track->GetSectionByIndex(i);
			message[offset+j] = ((char *)(&sec))[j];
		}
		offset += sizeof(Section);
	}
	
	for(i=0;i<track->GetNodeCount();i++)
	{
		unsigned long j;
		for(j=0;j<sizeof(Node);j++)
		{
			Node node = track->GetNodeByIndex(i);
			message[offset+j] = ((char *)(&node))[j];
		}
		offset += sizeof(Node);
	}

	for(i=0;i<track->GetSensorCount();i++)
	{
		unsigned long j;
		for(j=0;j<sizeof(Sensor);j++)
		{
			Sensor sensor = track->GetSensorByIndex(i);
			message[offset+j] = ((char *)(&sensor))[j];
		}
		offset += sizeof(Sensor);
	}
		
	bool ret = SendMessageToPeer(message, offset, SC_TRACK_DATA, destination, pipe);
	delete[] message;
	return ret;
}
	
bool TOS_SendTrackData(Track * track, char destination[][CLIENT_SIZE_CLIENTNAME], unsigned long destcnt, HANDLE pipe)
{
	/* Message Format */
	/* [# of sections (x4)][#of nodes (x4)][#of sensors(x4)][Sections].... [Nodes]....[sensors]...  */
	if(track==NULL)
		return false;
		
	if(destination == NULL)
		return false;
		
		char * message = new char[track->GetSectionCount() * sizeof(Section) + track->GetNodeCount() * sizeof(Node) + track->GetSensorCount() * sizeof(Sensor) + 16];
	
	if(message == NULL)
		return false;
		
	unsigned long offset = 0;
	
	message[0] = (unsigned char)(track->GetSectionCount() / 0x01000000);
	message[1] = (unsigned char)(track->GetSectionCount() / 0x00010000);
	message[2] = (unsigned char)(track->GetSectionCount() / 0x00000100);
	message[3] = (unsigned char)(track->GetSectionCount() % 0x00000100);
	
	message[4] = (unsigned char)(track->GetNodeCount() / 0x01000000);
	message[5] = (unsigned char)(track->GetNodeCount() / 0x00010000);
	message[6] = (unsigned char)(track->GetNodeCount() / 0x00000100);
	message[7] = (unsigned char)(track->GetNodeCount() % 0x00000100);

	message[8] = track->GetSensorCount() / 0x01000000;
	message[9] = track->GetSensorCount() / 0x00010000;
	message[10] = track->GetSensorCount() / 0x00010000;
	message[11] = track->GetSensorCount() % 0x00000100;
	
	offset=12;
	
	unsigned long i;
	for(i=0;i<track->GetSectionCount();i++)
	{
		unsigned long j;
		for(j=0;j<sizeof(Section);j++)
		{
			Section sec = track->GetSectionByIndex(i);
			message[offset+j] = ((char *)(&sec))[j];
		}
		offset += sizeof(Section);
	}
	
	for(i=0;i<track->GetNodeCount();i++)
	{
		unsigned long j;
		for(j=0;j<sizeof(Node);j++)
		{
			Node node = track->GetNodeByIndex(i);
			message[offset+j] = ((char *)(&node))[j];
		}
		offset += sizeof(Node);
	}

	for(i=0;i<track->GetSensorCount();i++)
	{
		unsigned long j;
		for(j=0;j<sizeof(Sensor);j++)
		{
			Sensor sensor = track->GetSensorByIndex(i);
			message[offset+j] = ((char *)(&sensor))[j];
		}
		offset += sizeof(Sensor);
	}
		
	bool ret = SendMulticastMessageToPeer(message, offset, SC_TRACK_DATA, destination, destcnt, pipe);
	delete[] message;
	return ret;
}

/* TRAIN MANAGER */
bool TOS_SetTrainSpeed(unsigned long TrainID, unsigned long Speed, bool direction, HANDLE pipe)
{
	/* Message Format */
	/* [Train ID (x2)][Speed (x4)][Direction(x1)] */
	
	char message[7];
	
	message[0] = (unsigned char)(TrainID / 256);
	message[1] = (unsigned char)(TrainID % 256);
	message[2] = (unsigned char)(Speed / 0x01000000);
	message[3] = (unsigned char)(Speed / 0x00010000);
	message[4] = (unsigned char)(Speed / 256);
	message[5] = (unsigned char)(Speed % 256);
	if(direction)
		message[6] = 1;
	else
		message[6] = 0;

	
	return SendMessageToPeer(message, 7, SC_SET_TRAIN_SPEED, TRAIN_MAN, pipe);
}
	
bool TOS_SetTrainAccessory(unsigned long TrainID, unsigned long Acc, unsigned long state, HANDLE pipe)
{
	/* Message Format */
	/* FUNCTION SKIPPED */
	return false;
}

bool TOS_RequestTrainUpdates(unsigned long interval, HANDLE pipe)
{
	/* Message Format */
	/* [interval in ms (x2)] */
	
	char message[2];
	
	message[0] = (unsigned char)(interval / 256);
	message[1] = (unsigned char)(interval % 256);
	
	return SendMessageToPeer(message, 2, SC_REQUEST_TRAIN_UPDATES, TRAIN_MAN, pipe);
}
	
bool TOS_SendTrainData(unsigned long traincnt, Train * trainlist, char * destination, HANDLE pipe)
{
	/* Message Format */
	/* [Train Count (x2)][TrainList [Id (x2)][Speed(x4)][Slope(x1)]
							[Section(x2)][Section Pos(x8)][X (x8)][Y (x8)] .....   ] */   
	
	/*
	unsigned long id;
	unsigned long speed;
	bool direction;
	bool slope;
	unsigned long section;
	double sectionpos;
	double x;
	double y;
	*/

	char * message = new char[2 + (34 * traincnt)];
	
	if(message == NULL)
		return false;
	
	unsigned long offset = 0;
	unsigned long i;
	
	message[0] = (unsigned char)(traincnt / 256);
	message[1] = (unsigned char)(traincnt % 256);
	offset = 2;
	for(i=0; i< traincnt; i++)
	{
		message[offset] = (unsigned char)(trainlist[i].id / 256);
		message[offset+1] = (unsigned char)(trainlist[i].id % 256);
			
		message[offset+2] = (unsigned char)(trainlist[i].speed / 0x01000000);
		message[offset+3] = (unsigned char)(trainlist[i].speed / 0x00010000);
		message[offset+4] = (unsigned char)(trainlist[i].speed / 256);
		message[offset+5] = (unsigned char)(trainlist[i].speed % 256);
		
		
			
		
		message[offset+6] = trainlist[i].slope;
			
		message[offset+7] = (unsigned char)(trainlist[i].section / 256);
		message[offset+8] = (unsigned char)(trainlist[i].section % 256);
		offset += 9;
		unsigned long j;
		for(j=0;j<8;j++)
		{
			message[offset+j] = ((unsigned char *)(&trainlist[i].sectionpos))[j];
		}
		offset += 8;
		for(j=0;j<8;j++)
		{
			message[offset+j] = ((unsigned char *)(&trainlist[i].x))[j];
		}
		offset += 8;
		for(j=0;j<8;j++)
		{
			message[offset+j] = ((unsigned char *)(&trainlist[i].y))[j];
		}
		offset += 8;
	}

	bool ret = SendMessageToPeer(message, offset, SC_TRAIN_DATA, destination, pipe);
	delete[] message;
	return ret;
	
}

bool TOS_SendTrainData(unsigned long traincnt, Train * trainlist, char destination[][CLIENT_SIZE_CLIENTNAME], unsigned long destcnt, HANDLE pipe)
{
	/* Message Format */
	/* [Train Count (x2)][TrainList [Id (x2)][Speed(x4)][Slope(x1)]
							[Section(x2)][Section Pos(x8)][X (x8)][Y (x8)] .....   ] */   
	
	/*
	unsigned long id;
	unsigned long speed;
	bool direction;
	bool slope;
	unsigned long section;
	double sectionpos;
	double x;
	double y;
	*/

	char * message = new char[2 + (34 * traincnt)];
	
	if(message == NULL)
		return false;
	
	unsigned long offset = 0;
	unsigned long i;
	
	message[0] = (unsigned char)(traincnt / 256);
	message[1] = (unsigned char)(traincnt % 256);
	offset = 2;
	for(i=0; i< traincnt; i++)
	{
		message[offset] = (unsigned char)(trainlist[i].id / 256);
		message[offset+1] = (unsigned char)(trainlist[i].id % 256);
			
		message[offset+2] = (unsigned char)(trainlist[i].speed / 0x01000000);
		message[offset+3] = (unsigned char)(trainlist[i].speed / 0x00010000);
		message[offset+4] = (unsigned char)(trainlist[i].speed / 256);
		message[offset+5] = (unsigned char)(trainlist[i].speed % 256);
		
		
			
		
		message[offset+6] = trainlist[i].slope;
			
		message[offset+7] = (unsigned char)(trainlist[i].section / 256);
		message[offset+8] = (unsigned char)(trainlist[i].section % 256);
		offset += 9;
		unsigned long j;
		for(j=0;j<8;j++)
		{
			message[offset+j] = ((unsigned char *)(&trainlist[i].sectionpos))[j];
		}
		offset += 8;
		for(j=0;j<8;j++)
		{
			message[offset+j] = ((unsigned char *)(&trainlist[i].x))[j];
		}
		offset += 8;
		for(j=0;j<8;j++)
		{
			message[offset+j] = ((unsigned char *)(&trainlist[i].y))[j];
		}
		offset += 8;
	}

	bool ret = SendMulticastMessageToPeer(message, offset, SC_TRAIN_DATA, destination, destcnt, pipe);
	delete[] message;
	return ret;
}
	
bool TOS_RequestTrainData(HANDLE pipe)
{

	return SendMessageToPeer(NULL, 0, SC_REQUEST_TRAIN_DATA, TRAIN_MAN, pipe);

}


bool TOS_AddTrain(unsigned long TrainID, unsigned long SectionID, char Slope, double SectionPos, HANDLE pipe)
{
	/* MessageID */
	/* [TrainID (x2)][SectionID(x2)][Slope(x1)][SectionPos(x8)] */
	
	char message[14];
	
	message[0] = TrainID / 256;
	message[1] = TrainID % 256;
	message[2] = SectionID / 256;
	message[3] = SectionID % 256;
	
	message[4] = Slope;
	
	unsigned long i;
	for(i=0; i<8; i++)
	{
		message[5+i] = ((char*)(&SectionPos))[i];
	}
	
	return SendMessageToPeer(message, 13, SC_ADD_TRAIN, TRAIN_MAN, pipe);
	
}
	
bool TOS_RemoveTrain(unsigned long TrainID, HANDLE pipe)
{
	/* MessageID */
	/* [TrainID (x2)] */
	
	char message[3];
	
	message[0] = TrainID / 256;
	message[1] = TrainID % 256;
	
	return SendMessageToPeer(message, 2, SC_REMOVE_TRAIN, TRAIN_MAN, pipe);
	
}


/* RESOURCE MANAGER */
bool TOS_RequestOwnership(unsigned long TrainID, unsigned char type, unsigned long ResourceID, HANDLE pipe)
{
	/* Message Format */
	/* [TrainID(x2)][Type(x1)][ResourceID(x2)] */
	
	char message[5];
	
	message[0] = TrainID / 256;
	message[1] = TrainID % 256;
	
	message[2] = type;
	
	message[3] = ResourceID / 256;
	message[4] = ResourceID % 256;
	
	return SendMessageToPeer(message, 5, SC_REQUEST_OWNERSHIP, RESOURCE_MAN, pipe);
	
}

bool TOS_ReleaseOwnership(unsigned char type, unsigned long ResourceID, HANDLE pipe)
{
	/* message format*/
	/*[type(x1)][id(x2)]*/

	char message[3];
	
	message[0] = type;
	
	message[1] = ResourceID / 256;
	message[2] = ResourceID % 256;
	
	return SendMessageToPeer(message, 3, SC_RELEASE_OWNERSHIP, RESOURCE_MAN, pipe);

}


bool TOS_ActivateNode(unsigned long NodeID, HANDLE pipe)
{
	/* Message Format */
	/* [NodeId(x2)] */
	
	char message[2];
	
	message[0] = NodeID / 256;
	message[1] = NodeID % 256;
	
	return SendMessageToPeer(message, 2, SC_ACTIVATE_NODE, RESOURCE_MAN, pipe);
	
}
	
bool TOS_RequestResourceUpdates(HANDLE pipe)
{
	/* Message Format */
	/*	
	char message[2];
	
	message[0] = Interval / 256;
	message[1] = Interval % 256;
	*/
	return SendMessageToPeer(NULL, 0, SC_REQUEST_RESOURCE_UPDATES, RESOURCE_MAN, pipe);
	
}
	
bool TOS_GrantResource(unsigned long TrainID, unsigned char type, unsigned long ResourceID, char * destination, HANDLE pipe)
{
	/* Message Format */
	/* [TrainID(x2)][Type(x1)][ResourceID(x2)] */
	
	char message[5];
	
	message[0] = TrainID / 256;
	message[1] = TrainID % 256;
	
	message[2] = type;
	
	message[3] = ResourceID / 256;
	message[4] = ResourceID % 256;
	
	return SendMessageToPeer(message, 5, SC_RESOURCE_GRANT, destination, pipe);
	
}
	
bool TOS_DenyResource(unsigned long TrainID,unsigned char type, unsigned long ResourceID, char * destination, HANDLE pipe)
{
	/* Message Format */
	/* [TrainID(x2)][Type(x1)][ResourceID(x2)] */
	
	char message[5];
	
	message[0] = TrainID / 256;
	message[1] = TrainID % 256;
	
	message[2] = type;
	
	message[3] = ResourceID / 256;
	message[4] = ResourceID % 256;
	
	return SendMessageToPeer(message, 5, SC_RESOURCE_DENY, destination, pipe);
	
}

bool TOS_SendResourceData(unsigned long cnt, Resource * res, char * destination, HANDLE pipe)
{
	/* Message Format */
	/* [cnt (x2)][Resource[Type(x1)][ID(x2)][Owner(x2)][Active(x1)]] ........  */
	
	char * message = new char[2+(cnt*8)];
	
	if(message==NULL)
	{ 
		return false;
	}
	
	message[0] = cnt / 256;
	message[1] = cnt % 256;
	unsigned long offset = 2;
	unsigned long i;
	for(i=0;i<cnt;i++)
	{
		message[offset] 	= res[i].type;
		message[offset+1] 	= res[i].id / 256;
		message[offset+2] 	= res[i].id % 256;
		message[offset+3]	= res[i].owner / 0x01000000;
		message[offset+4]	= res[i].owner / 0x00010000;
		message[offset+5]	= res[i].owner / 256;
		message[offset+6]	= res[i].owner % 256;
		message[offset+7]	= res[i].active;
		offset += 8;
	}
	
	bool ret = SendMessageToPeer(message, offset, SC_RESOURCE_DATA, destination, pipe);
	delete[] message;
	return ret;
}
	
bool TOS_SendResourceData(unsigned long cnt, Resource * res, char destination[][CLIENT_SIZE_CLIENTNAME], unsigned long destcnt, HANDLE pipe)
{
	/* Message Format */
	/* [cnt (x2)][Resource[Type(x1)][ID(x2)][Owner(x4)][Active(x1)]] ........  */
	
	char * message = new char[2+(cnt*8)];
	
	if(message==NULL)
	{ 
		return false;
	}
	
	message[0] = cnt / 256;
	message[1] = cnt % 256;
	unsigned long offset = 2;
	unsigned long i;
	for(i=0;i<cnt;i++)
	{
		message[offset] 	= res[i].type;
		message[offset+1] 	= res[i].id / 256;
		message[offset+2] 	= res[i].id % 256;
		message[offset+3]	= res[i].owner / 0x01000000;
		message[offset+4]	= res[i].owner / 0x00010000;
		message[offset+5]	= res[i].owner / 256;
		message[offset+6]	= res[i].owner % 256;
		message[offset+7]	= res[i].active;
		offset += 8;
	}
	
	bool ret = SendMulticastMessageToPeer(message, offset, SC_RESOURCE_DATA, destination, destcnt, pipe);
	delete[] message;
	return ret;
}

/* DCC MANAGER */
bool TOS_SetTrainSpeedDCC(unsigned long TrainAddress, unsigned long Speed, unsigned char direction, HANDLE pipe)
{
	/* message format */
	/* [DCC address (x2)][Speed(x2)][direction(x1)] */
	char message[6];
	
	message[0] = TrainAddress / 256;
	message[1] = TrainAddress % 256;
	message[2] = Speed / 256;
	message[3] = Speed % 256;
	message[4] = direction;
	
	return SendMessageToPeer(message, 5, SC_SET_TRAIN_SPEED_DCC, DCC_MAN, pipe);
}
	
bool TOS_SetTrainAccessoryDCC(unsigned long TrainAddress, unsigned long Accessory, unsigned long value, HANDLE pipe)
{
	/* message format */
	/* [dcc address(x2)][accessroy(x2)][value(x1)] */
	
	char message[6];
	
	message[0] = TrainAddress / 256;
	message[1] = TrainAddress % 256;
	message[2] = Accessory / 256;
	message[3] = Accessory % 256;
	message[4] = value;
	
	return SendMessageToPeer(message, 5, SC_SET_TRAIN_ACCESSORY_DCC, DCC_MAN, pipe);
}
	
bool TOS_SetTurnout(unsigned long Address, unsigned char value, HANDLE pipe)
{
	/* message format */
	/* [dcc address(x2)][value(x1)] */
	
	char message[4];
	
	message[0] = Address / 256;
	message[1] = Address % 256;
	message[2] = value;
	
	return SendMessageToPeer(message, 3, SC_SET_TURNOUT, DCC_MAN, pipe);
}

bool TOS_ReadAIU(unsigned long Address, unsigned long Input, HANDLE pipe)
{
	/*message format */
	/* [address(x2)][input(x1)] */
	
	char message[4];
	
	message[0] = Address / 256;
	message[1] = Address % 256;
	message[2] = Input;
	
	return SendMessageToPeer(message, 3, SC_READ_AIU, DCC_MAN, pipe);
}

bool TOS_SendAIUUpdate(unsigned long SensorIndex, bool value, char * destination, HANDLE pipe)
{
	/* message format */
	/* [address(x2)][value(x1)] */
	
	char message[5];
	
	message[0] = SensorIndex / 256;
	message[1] = SensorIndex % 256;
	message[2] = value;
	
	return SendMessageToPeer(message, 3, SC_AIU_UPDATES, destination, pipe);
}

bool TOS_SendAIUUpdate(unsigned long SensorIndex, bool value, char destination[][CLIENT_SIZE_CLIENTNAME], unsigned long destcnt, HANDLE pipe)
{
	/* message format */
	/* [address(x2)][value(x1)] */
	
	char message[5];
	
	message[0] = SensorIndex / 256;
	message[1] = SensorIndex % 256;
	message[2] = value;
	
	return SendMulticastMessageToPeer(message, 3, SC_AIU_UPDATES, destination, destcnt, pipe);
}

bool TOS_SendDCCPacket(unsigned long length, char * data, HANDLE pipe)
{
	/*message format */
	/* [length(x2)][data(x?)] */
	
	char * message = new char[length];
	if(message==NULL)
		return false;
		
	unsigned long i;
	for(i=0;i<length;i++)
	{
		message[i] = data[i];
	}
	
	bool ret = SendMessageToPeer(message, length, SC_SEND_DCC_PACKET, DCC_MAN, pipe);
	delete[] message;
	
	return ret;
}

bool TOS_DCCAck(char * destination, HANDLE pipe)
{
	return false;
}

bool TOS_DCCNack(unsigned long error, char * destination, HANDLE pipe)
{
	return false;
}


/* Decoding Track DATA */
bool TOS_ReadTrackData(Track * track, char * message, unsigned long length)
{
	/* Message Format */
	/* [# of sections (x4)][#of nodes (x4)][#of nodes(x4)][Sections].... [Nodes]....[sensors]...  */
	if(track==NULL)
		return false;
		
	if(message == NULL)
		return false;

	if(length < 12)
		return false;
		
	unsigned long offset = 0;
	unsigned long sections, nodes, sensors;

	track->Clear();
	
	sections = (unsigned char)message[0] * 0x01000000;
	sections += (unsigned char)message[1] * 0x00010000;
	sections += (unsigned char)message[2] * 0x00000100;
	sections += (unsigned char)message[3];
	
	nodes = (unsigned char)message[4] * 0x01000000;
	nodes += (unsigned char)message[5] * 0x00010000;
	nodes += (unsigned char)message[6] * 0x00000100;
	nodes += (unsigned char)message[7];

	sensors = (unsigned char)message[8] * 0x01000000;
	sensors += (unsigned char)message[9] * 0x00010000;
	sensors += (unsigned char)message[10] * 0x00000100;
	sensors += (unsigned char)message[11];
	
	offset=12;
	
	unsigned long i;
	for(i=0;i<sections;i++)
	{
		Section sec ;
		unsigned long j;
		for(j=0;j<sizeof(Section);j++)
		{
			((char *)(&sec))[j] = message[offset+j];
		}
		track->AddSection(sec);
		offset += sizeof(Section);
	}
	
	for(i=0;i<nodes;i++)
	{
		Node n ;
		unsigned long j;
		for(j=0;j<sizeof(Node);j++)
		{
			((char *)(&n))[j] = message[offset+j];
		}
		track->AddNode(n);
		offset += sizeof(Node);
	}

	for(i=0;i<sensors;i++)
	{
		Sensor s; 
		unsigned long j;
		for(j=0;j<sizeof(Sensor);j++)
		{
			((char *)(&s))[j] = message[offset+j];
		}
		track->AddSensor(s);
		offset += sizeof(Sensor);
	}

	return true;
}

bool TOS_InitDCC(char * port, HANDLE pipe)
{
	return SendMessageToPeer(port, strlen(port), SC_INIT_DCC, DCC_MAN, pipe);
}

bool TOS_DCCReady(char * destination, HANDLE pipe)
{
	return SendMessageToPeer(NULL, 0, SC_DCC_READY, destination, pipe);
}

bool TOS_DCCNotReady(char * destination, HANDLE pipe)
{
	return SendMessageToPeer(NULL, 0, SC_DCC_NOT_READY, destination, pipe);
}

bool TOS_DCCReady(char  destination[][CLIENT_SIZE_CLIENTNAME], unsigned long destcnt, HANDLE pipe)
{
	return SendMulticastMessageToPeer(NULL, 0, SC_DCC_READY, destination, destcnt, pipe);
}


bool TOS_RequestDCCUpdates(HANDLE pipe)
{
	return SendMessageToPeer(NULL, 0, SC_REQUEST_DCC_UPDATES, DCC_MAN, pipe);
}

bool TOS_ReadAIUUpdate(char * data, unsigned long length, unsigned long& index, bool& value)
{
	
	/* message format */
	/* [address(x2)][value(x1)] */
	if(length != 3)
		return false;

	index = data[0] * 256;
	index += data[1]; 

	value = data[2];

	return true;

}


bool TOS_ReadResourceData(unsigned long& myres, unsigned long& length, char * data, unsigned long dlength )
{
	/* Message Format */
	/* [cnt (x2)][Resource[Type(x1)][ID(x2)][Owner(x4)][Active(x1)]] ........  */
	
	
	/* 1 - is section */
	/* 2 - is node    */

	Resource * res;
	
	if(data==NULL)
	{ 
		return false;
	}
	
	length = data[0] * 256;
	length += data[1];
	
	if(length > 16000)
	{
		data = NULL;
		length = 0;
		return false;
	}
	
	res = new Resource[length];
	
	if( data == NULL)
	{
		length = 0;
		return false;
	}
		
	
	unsigned long offset = 2;
	unsigned long i;
	for(i=0;i<length;i++)
	{
		if((offset + 5) > dlength)
		{
			/* data length mismatch */
			delete data;
			data = NULL;
			length = 0;
			return false;
		}
			
		res[i].type			= data[offset];
		res[i].id			= data[offset+1] * 256;
		res[i].id			+= data[offset+2];
		res[i].owner		= (unsigned char)data[offset+3] * 0x01000000;
		res[i].owner		+= (unsigned char)data[offset+4] * 0x00010000; 
		res[i].owner		+= (unsigned char)data[offset+5] * 256;
		res[i].owner		+= (unsigned char)data[offset+6];
		res[i].active		= data[offset+7];
		offset += 8;
	}
	
	myres = (unsigned long)res;

	return true;
}




bool TOS_ReadTrainData(unsigned long& trainlist, unsigned long& traincnt, char * data, unsigned long length)
{
	/* Message Format */
	/* [Train Count (x2)][TrainList [Id (x2)][Speed(x4)][Slope(x1)]
							[Section(x2)][Section Pos(x8)][X (x8)][Y (x8)] .....   ] */   
	
	/*
	unsigned long id;
	unsigned long speed;
	bool direction;
	bool slope;
	unsigned long section;
	double sectionpos;
	double x;
	double y;
	*/

	trainlist = 0;

	Train * tlist;

	if(data == NULL)
		return false;

	if(length < 2)
		return false;
	
	unsigned long offset = 0;
	unsigned long i;
	

	traincnt = data[0] * 256;
	traincnt += data[1];


	tlist = new Train[traincnt];
	
	if(tlist == NULL)
		return false;



	offset = 2;
	for(i=0; i< traincnt; i++)
	{
		tlist[i].id = data[offset] * 256;
		tlist[i].id += data[offset+1];
			
		tlist[i].speed = (unsigned long)(data[offset+2] * 0x01000000); 
		tlist[i].speed += (unsigned long)(data[offset+3] * 0x00010000);
		tlist[i].speed += (unsigned long)(data[offset+4] * 0x00000100);
		tlist[i].speed += (unsigned long)(data[offset+5]);
		
		tlist[i].slope = data[offset+6];
			
		tlist[i].section = (unsigned long)(data[offset+7]*256);
		tlist[i].section = (unsigned long)(data[offset+8]);
		
		offset += 9;
		unsigned long j;
		for(j=0;j<8;j++)
		{
			((unsigned char *)(&(tlist[i].sectionpos)))[j] = data[offset+j];
		}
		offset += 8;
		for(j=0;j<8;j++)
		{			
			((unsigned char *)(&(tlist[i].x)))[j] = data[offset+j];
		}
		offset += 8;
		for(j=0;j<8;j++)
		{
			((unsigned char *)((&tlist[i].y)))[j] = data[offset+j];
		}
		offset += 8;
	}

	trainlist = (unsigned long)tlist;
	
	return true;
}






