/***************************************************/
/* API CALLS SPECIFIC TO THE TOS SYSTEM            */
/*                                                 */
/*                                                 */ 
/*                                                 */
/*                                                 */
/*                                                 */
/*                                                 */
/*                                                 */
/*                                                 */
/*  CHRISTOPHER KRATZ SNC 4/3/2007                 */
/***************************************************/
/* TOS_CLIENT.H */

#ifndef TOSAPI
#define TOSAPI

/* CONSTANTS */
char TRACK_MAN[9] 		= "TrackMan";
char TRAIN_MAN[9] 		= "TrainMan";
char RESOURCE_MAN[12] 	= "ResourceMan";
char DCC_MAN[9] 		= "DCCMan";
char UI[9] 				= "TosUI";

/* TRACK MANAGER */
const unsigned long SC_REQUEST_TRACK_DATA 			= 0x1001;  //<<--
const unsigned long SC_REQUEST_TRACK_DATA_UPDATES 	= 0x1002;  //<<--
const unsigned long SC_LOAD_TRACK_FROM_FILE 		= 0x1003;  //<<--
const unsigned long SC_TRACK_DATA					= 0x1004;  //-->>

/* TRAIN MANAGER */
const unsigned long SC_SET_TRAIN_SPEED				= 0x1101;  //<<-- 
const unsigned long SC_SET_TRAIN_ACCESSORY			= 0x1102;  //<<--
const unsigned long SC_REQUEST_TRAIN_UPDATES		= 0x1103;  //<<--
const unsigned long SC_TRAIN_DATA					= 0x1104;  //-->>
const unsigned long SC_REQUEST_TRAIN_DATA			= 0x1105;  //<<--
const unsigned long SC_ADD_TRAIN					= 0x1106;  //<<--
const unsigned long SC_REMOVE_TRAIN					= 0x1107;  //<<--


/* RESOURCE MANAGER */
const unsigned long SC_REQUEST_OWNERSHIP			= 0x1201;  //<<--
const unsigned long SC_ACTIVATE_NODE				= 0x1202;  //<<--
const unsigned long SC_REQUEST_RESOURCE_UPDATES		= 0x1203;  //<<--
const unsigned long SC_RESOURCE_GRANT				= 0x1204;  //-->>
const unsigned long SC_RESOURCE_DENY				= 0x1205;  //-->>
const unsigned long SC_RESOURCE_DATA				= 0x1206;  //-->>
const unsigned long SC_RELEASE_OWNERSHIP			= 0x1207;  //<<--

/* DCC MANAGER */
const unsigned long SC_SET_TRAIN_SPEED_DCC			= 0x1301;  //<<--
const unsigned long SC_SET_TRAIN_ACCESSORY_DCC		= 0x1302;  //<<--
const unsigned long SC_SET_TURNOUT					= 0x1303;  //<<--
const unsigned long SC_READ_AIU						= 0x1304;  //<<--
const unsigned long SC_REQUEST_DCC_UPDATES			= 0x1305;  //<<--
const unsigned long SC_AIU_UPDATES					= 0x1306;  //-->>
const unsigned long SC_SEND_DCC_PACKET				= 0x1307;  //-->>
const unsigned long SC_DCC_ACK						= 0x1308;  //-->>
const unsigned long SC_DCC_NACK						= 0x1309;  //-->>
const unsigned long SC_INIT_DCC						= 0x1310;  //<<--
const unsigned long SC_DCC_READY					= 0x1311;  //-->>
const unsigned long SC_DCC_NOT_READY				= 0x1312;  //-->>


/* TRACK MANAGER RELATED FUNCTIONS */

bool TOS_RequestTrackData(HANDLE pipe);
bool TOS_RequestTrackDataUpdates(HANDLE pipe);
bool TOS_RequestLoadTrackFromFile(char * filename,HANDLE pipe);
bool TOS_SendTrackData(Track * track, char * destination,HANDLE pipe);
bool TOS_SendTrackData(Track * track, char destination[][CLIENT_SIZE_CLIENTNAME], unsigned long destcnt,HANDLE pipe);

/* TRAIN MANAGER */
bool TOS_SetTrainSpeed(unsigned long TrainID, unsigned long Speed, bool direction,HANDLE pipe);
bool TOS_SetTrainAccessory(unsigned long TrainID, unsigned long Acc, unsigned long state,HANDLE pipe );
bool TOS_RequestTrainUpdates(unsigned long interval, HANDLE pipe);
bool TOS_SendTrainData(unsigned long traincnt, Train * trainlist, char * destination, HANDLE pipe);
bool TOS_SendTrainData(unsigned long traincnt, Train * trainlist, char destination[][CLIENT_SIZE_CLIENTNAME], unsigned long destcnt, HANDLE pipe);
bool TOS_RequestTrainData(HANDLE pipe);
bool TOS_AddTrain(unsigned long TrainID, unsigned long SectionID, char Slope, double SectionPos, HANDLE pipe);
bool TOS_RemoveTrain(unsigned long TrainID, HANDLE pipe);

/* RESOURCE MANAGER */
bool TOS_RequestOwnership(unsigned long TrainID, unsigned char type, unsigned long ResourceID, HANDLE pipe);
bool TOS_ReleaseOwnership(unsigned char type, unsigned long ResourceID, HANDLE pipe);
bool TOS_ActivateNode(unsigned long NodeID, HANDLE pipe);
bool TOS_RequestResourceUpdates(HANDLE pipe);
bool TOS_GrantResource(unsigned long TrainID,unsigned char type, unsigned long ResourceID, char * destination, HANDLE pipe);
bool TOS_DenyResource(unsigned long TrainID,unsigned char type, unsigned long ResourceID, char * destination, HANDLE pipe);
bool TOS_SendResourceData(unsigned long cnt, Resource * res, char * destination, HANDLE pipe);
bool TOS_SendResourceData(unsigned long cnt, Resource * res, char destination[][CLIENT_SIZE_CLIENTNAME], unsigned long destcnt, HANDLE pipe);

/* DCC MANAGER */
bool TOS_SetTrainSpeedDCC(unsigned long TrainAddress, unsigned long Speed, unsigned char direction, HANDLE pipe);
bool TOS_SetTrainAccessoryDCC(unsigned long TrainAddress, unsigned long Accessory, unsigned long value, HANDLE pipe);
bool TOS_SetTurnout(unsigned long Address, unsigned char value, HANDLE pipe);
bool TOS_ReadAIU(unsigned long Address, unsigned long Input, HANDLE pipe);
bool TOS_SendAIUUpdate(unsigned long SensorIndex, bool value, char * destination, HANDLE pipe);
bool TOS_SendAIUUpdate(unsigned long SensorIndex, bool value, char destination[][CLIENT_SIZE_CLIENTNAME], unsigned long destcnt, HANDLE pipe);
bool TOS_SendDCCPacket(unsigned long length, char * data, HANDLE pipe);
bool TOS_DCCAck(char * destination, HANDLE pipe);
bool TOS_DCCNack(unsigned long error, char * destination, HANDLE pipe);
bool TOS_InitDCC(char * port, HANDLE pipe);
bool TOS_DCCReady(char * destination, HANDLE pipe);
bool TOS_DCCNotReady(char * destination, HANDLE pipe);
bool TOS_RequestDCCUpdates(HANDLE pipe);



bool TOS_ReadTrackData(Track * track, char * message, unsigned long length);
bool TOS_ReadAIUUpdate(char * data, unsigned long length, unsigned long& index, bool& value);
bool TOS_ReadResourceData(unsigned long& myres, unsigned long& length, char * data, unsigned long dlength );
bool TOS_ReadTrainData(unsigned long& trainlist, unsigned long& traincnt, char * data, unsigned long length);

#endif




